/*********************************************************************************************************************
* TC264 Opensourec Library 即（TC264 开源库）是一个基于官方 SDK 接口的第三方开源库
* Copyright (c) 2022 SEEKFREE 逐飞科技
*
* 本文件是 TC264 开源库的一部分
*
* TC264 开源库 是免费软件
* 您可以根据自由软件基金会发布的 GPL（GNU General Public License，即 GNU通用公共许可证）的条款
* 即 GPL 的第3版（即 GPL3.0）或（您选择的）任何后来的版本，重新发布和/或修改它
*
* 本开源库的发布是希望它能发挥作用，但并未对其作任何的保证
* 甚至没有隐含的适销性或适合特定用途的保证
* 更多细节请参见 GPL
*
* 您应该在收到本开源库的同时收到一份 GPL 的副本
* 如果没有，请参阅<https://www.gnu.org/licenses/>
*
* 额外注明：
* 本开源库使用 GPL3.0 开源许可证协议 以上许可申明为译文版本
* 许可申明英文版在 libraries/doc 文件夹下的 GPL3_permission_statement.txt 文件中
* 许可证副本在 libraries 文件夹下 即该文件夹下的 LICENSE 文件
* 欢迎各位使用并传播本程序 但修改内容时必须保留逐飞科技的版权声明（即本声明）
*
* 文件名称          zf_device_ble6a20
* 公司名称          成都逐飞科技有限公司
* 版本信息          查看 libraries/doc 文件夹内 version 文件 版本说明
* 开发环境          ADS v1.10.2
* 适用平台          TC264D
* 店铺链接          https://seekfree.taobao.com/
*
* 修改记录
* 日期              作者               备注
* 2024-07-02        pudding            first version
********************************************************************************************************************/
/*********************************************************************************************************************
* 接线定义：
*                   ------------------------------------
*                   模块管脚            单片机管脚
*                   RX                  查看 zf_device_ble6a20.h 中 BLE6A20_RX_PIN  宏定义
*                   TX                  查看 zf_device_ble6a20.h 中 BLE6A20_TX_PIN  宏定义
*                   RTS                 查看 zf_device_ble6a20.h 中 BLE6A20_RTS_PIN 宏定义
*                   VCC                 3.3V电源
*                   GND                 电源地
*                   其余引脚悬空
*                   ------------------------------------
*********************************************************************************************************************/

#include "zf_common_clock.h"
#include "zf_common_debug.h"
#include "zf_common_fifo.h"
#include "zf_driver_delay.h"
#include "zf_driver_gpio.h"
#include "zf_driver_uart.h"
#include "zf_device_type.h"
#include "zf_device_ble6a20.h"

static  fifo_struct                                     ble6a20_fifo;
static  uint8                                           ble6a20_buffer[BLE6A20_BUFFER_SIZE];

static          uint8                                   ble6a20_data            = 0;
#if (1 == BLE6A20_AUTO_BAUD_RATE)
static volatile ble6a20_auto_baudrate_state_enum       ble6a20_auto_baud_flag     = BLE6A20_AUTO_BAUD_RATE_INIT;
static volatile uint8                                  ble6a20_auto_baud_data[3]  = {0x00, 0x01, 0x03};
#endif

//-------------------------------------------------------------------------------------------------------------------
// 函数简介     蓝牙转串口模块 发送数据
// 参数说明     data            8bit 数据
// 返回参数     uint32          剩余发送长度 0-发送完毕 1-未发送完成
// 使用示例     ble6a20_send_byte(data);
// 备注信息
//-------------------------------------------------------------------------------------------------------------------
uint32 ble6a20_send_byte (const uint8 data)
{
    uint16 time_count = BLE6A20_TIMEOUT_COUNT;
    while(time_count)
    {
        if(!gpio_get_level(BLE6A20_RTS_PIN))
        {
            uart_write_byte(BLE6A20_INDEX, data);                         // 发送数据
            break;
        }
        time_count --;
        system_delay_ms(1);
    }
    return (0 == time_count);
}

//-------------------------------------------------------------------------------------------------------------------
// 函数简介     蓝牙转串口模块 发送数据块
// 参数说明     *buff           发送缓冲区
// 参数说明     len             发送数据长度
// 返回参数     uint32          剩余发送长度
// 使用示例     ble6a20_send_buffer(buff, 64);
// 备注信息
//-------------------------------------------------------------------------------------------------------------------
uint32 ble6a20_send_buffer (const uint8 *buff, uint32 len)
{
    zf_assert(NULL != buff);
    uint16 time_count = 0;
    while(0 != len)
    {
        if(!gpio_get_level(BLE6A20_RTS_PIN))                                    // 如果RTS为低电平 则继续发送数据
        {
            if(30 <= len)                                                       // 数据分 30byte 每包发送
            {
                uart_write_buffer(BLE6A20_INDEX, buff, 30);                     // 发送数据
                buff += 30;                                                     // 地址偏移
                len -= 30;                                                      // 数量
                time_count = 0;
            }
            else                                                                // 不足 30byte 的数据一次性发送完毕
            {
                uart_write_buffer(BLE6A20_INDEX, buff, len);                    // 发送数据
                len = 0;
                break;
            }
        }
        else                                                                    // 如果RTS为高电平 则模块忙
        {
            if(BLE6A20_TIMEOUT_COUNT <= (++ time_count))                        // 超出了最大等待时间
            {
                break;                                                          // 退出发送
            }
            system_delay_ms(1);
        }
    }
    return len;
}

//-------------------------------------------------------------------------------------------------------------------
// 函数简介     蓝牙转串口模块 发送字符串
// 参数说明     *str            要发送的字符串地址
// 返回参数     uint32          剩余发送长度
// 使用示例     ble6a20_send_string("Believe in yourself.");
// 备注信息
//-------------------------------------------------------------------------------------------------------------------
uint32 ble6a20_send_string (const char *str)
{
    zf_assert(NULL != str);
    uint16 time_count = 0;
    uint32 len = strlen(str);
    while(0 != len)
    {
        if(!gpio_get_level(BLE6A20_RTS_PIN))                              // 如果RTS为低电平 则继续发送数据
        {
            if(128 <= len)                                                       // 数据分 128byte 每包发送
            {
                uart_write_buffer(BLE6A20_INDEX, (const uint8 *)str, 128); // 发送数据
                str += 128;                                                      // 地址偏移
                len -= 128;                                                      // 数量
                time_count = 0;
            }
            else                                                                // 不足 128byte 的数据一次性发送完毕
            {
                uart_write_buffer(BLE6A20_INDEX, (const uint8 *)str, len);// 发送数据
                len = 0;
                break;
            }
        }
        else                                                                    // 如果RTS为高电平 则模块忙
        {
            if(BLE6A20_TIMEOUT_COUNT <= (++ time_count))                  // 超出了最大等待时间
            {
                break;                                                          // 退出发送
            }
            system_delay_ms(1);
        }
    }
    return len;
}

//-------------------------------------------------------------------------------------------------------------------
// 函数简介     蓝牙转串口模块 发送摄像头图像至上位机查看图像
// 参数说明     *image_addr     需要发送的图像地址
// 参数说明     image_size      图像的大小
// 返回参数     void
// 使用示例     ble6a20_send_image(&mt9v03x_image[0][0], MT9V03X_IMAGE_SIZE);
// 备注信息
//-------------------------------------------------------------------------------------------------------------------
void ble6a20_send_image (const uint8 *image_addr, uint32 image_size)
{
    zf_assert(NULL != image_addr);
    extern uint8 camera_send_image_frame_header[4];
    ble6a20_send_buffer(camera_send_image_frame_header, 4);
    ble6a20_send_buffer((uint8 *)image_addr, image_size);
}

//-------------------------------------------------------------------------------------------------------------------
// 函数简介     蓝牙转串口模块 读取缓冲区数据
// 参数说明     *buff           接收缓冲区
// 参数说明     len             读取数据长度
// 返回参数     uint32          实际读取数据长度
// 使用示例     ble6a20_read_buffer(buff, 32);
// 备注信息
//-------------------------------------------------------------------------------------------------------------------
uint32 ble6a20_read_buffer (uint8 *buff, uint32 len)
{
    zf_assert(NULL != buff);
    uint32 data_len = len;
    fifo_read_buffer(&ble6a20_fifo, buff, &data_len, FIFO_READ_AND_CLEAN);
    return data_len;
}

//-------------------------------------------------------------------------------------------------------------------
// 函数简介     蓝牙转串口模块 串口中断回调函数
// 参数说明     void
// 返回参数     void
// 使用示例     ble6a20_callback();
// 备注信息     该函数在 ISR 文件 串口中断程序被调用
//              由串口中断服务函数调用 wireless_module_uart_handler() 函数
//              再由 wireless_module_uart_handler() 函数调用本函数
//-------------------------------------------------------------------------------------------------------------------
void ble6a20_callback (void)
{
    if(uart_query_byte(BLE6A20_INDEX, &ble6a20_data))
    {
        fifo_write_buffer(&ble6a20_fifo, &ble6a20_data, 1);
#if BLE6A20_AUTO_BAUD_RATE                                                // 开启自动波特率
        if(BLE6A20_AUTO_BAUD_RATE_START == ble6a20_auto_baud_flag && 3 == fifo_used(&ble6a20_fifo))
        {
            uint32 ble6a20_auto_baud_count = 3;
            ble6a20_auto_baud_flag = BLE6A20_AUTO_BAUD_RATE_GET_ACK;
            fifo_read_buffer(&ble6a20_fifo, (uint8 *)ble6a20_auto_baud_data, (uint32 *)&ble6a20_auto_baud_count, FIFO_READ_AND_CLEAN);
        }
#endif
    }
}

//-------------------------------------------------------------------------------------------------------------------
// 函数简介     蓝牙转串口模块 初始化
// 参数说明     void
// 返回参数     void
// 使用示例     ble6a20_init();
// 备注信息
//-------------------------------------------------------------------------------------------------------------------
uint8 ble6a20_init (void)
{
    uint8 return_state = 0;

    // 等待模块初始化
    system_delay_ms(50);

    set_wireless_type(BLE6A20, ble6a20_callback);

    fifo_init(&ble6a20_fifo, FIFO_DATA_8BIT, ble6a20_buffer, BLE6A20_BUFFER_SIZE);
    gpio_init(BLE6A20_RTS_PIN, GPI, GPIO_HIGH, GPI_PULL_UP);
#if(0 == BLE6A20_AUTO_BAUD_RATE)                                                        // 关闭自动波特率
    // 本函数使用的波特率为115200 为蓝牙转串口模块的默认波特率 如需其他波特率请自行配置模块并修改串口的波特率
    uart_init (BLE6A20_INDEX, BLE6A20_BUAD_RATE, BLE6A20_RX_PIN, BLE6A20_TX_PIN);       // 初始化串口
    uart_rx_interrupt(BLE6A20_INDEX, 1);
#elif(1 == BLE6A20_AUTO_BAUD_RATE)                                                      // 开启自动波特率
    uint8 rts_init_status = 0;

    ble6a20_auto_baud_flag = BLE6A20_AUTO_BAUD_RATE_INIT;
    ble6a20_auto_baud_data[0] = 0;
    ble6a20_auto_baud_data[1] = 1;
    ble6a20_auto_baud_data[2] = 3;

    rts_init_status = gpio_get_level(BLE6A20_RTS_PIN);
    gpio_init(BLE6A20_RTS_PIN, GPO, rts_init_status, GPO_PUSH_PULL);                    // 初始化流控引脚

    uart_init (BLE6A20_INDEX, BLE6A20_BUAD_RATE, BLE6A20_RX_PIN, BLE6A20_TX_PIN);       // 初始化串口
    uart_rx_interrupt(BLE6A20_INDEX, 1);

    system_delay_ms(5);                                                                 // 模块上电之后需要延时等待
    gpio_set_level(BLE6A20_RTS_PIN, !rts_init_status);                                  // RTS引脚拉高，进入自动波特率模式
    system_delay_ms(100);                                                               // RTS拉高之后必须延时20ms
    gpio_toggle_level(BLE6A20_RTS_PIN);                                                 // RTS引脚取反

    do
    {
        ble6a20_auto_baud_flag = BLE6A20_AUTO_BAUD_RATE_START;
        uart_write_byte(BLE6A20_INDEX, ble6a20_auto_baud_data[0]);                      // 发送特定数据 用于模块自动判断波特率
        uart_write_byte(BLE6A20_INDEX, ble6a20_auto_baud_data[1]);                      // 发送特定数据 用于模块自动判断波特率
        uart_write_byte(BLE6A20_INDEX, ble6a20_auto_baud_data[2]);                      // 发送特定数据 用于模块自动判断波特率
        system_delay_ms(20);

        if(BLE6A20_AUTO_BAUD_RATE_GET_ACK != ble6a20_auto_baud_flag)                    // 检验自动波特率是否完成
        {
            return_state = 1;                                                           // 如果程序进入到此语句内 说明自动波特率失败了
            break;
        }

        if( 0xa5 != ble6a20_auto_baud_data[0] &&                                        // 检验自动波特率是否正确
            0xff != ble6a20_auto_baud_data[1] &&                                        // 检验自动波特率是否正确
            0xff != ble6a20_auto_baud_data[2] )                                         // 检验自动波特率是否正确
        {
            return_state = 1;                                                           // 如果程序进入到此语句内 说明自动波特率失败了
            break;
        }
        ble6a20_auto_baud_flag = BLE6A20_AUTO_BAUD_RATE_SUCCESS;

        gpio_init(BLE6A20_RTS_PIN, GPI, 0, GPI_PULL_UP);                                // 初始化流控引脚
        system_delay_ms(10);                                                            // 延时等待 模块准备就绪
    }while(0);
#endif
    return return_state;
}
